<?php

namespace Drupal\consent_manager\Form;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The base form for the consent_manager submodules settings form.
 */
abstract class SettingsBaseForm extends ConfigFormBase {

  /**
   * Constructs a ConsentManagerCmpSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\Core\Extension\ModuleExtensionList $moduleExtensionList
   *   The module extension list.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    protected LanguageManagerInterface $languageManager,
    protected ModuleExtensionList $moduleExtensionList,
    $typedConfigManager = NULL,
  ) {
    parent::__construct($config_factory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('language_manager'),
      $container->get('extension.list.module'),
      $container->get('config.typed'),
    );
  }

  /**
   * Gets plugin type.
   *
   * @return string
   *   The plugin type.
   */
  abstract protected function getPluginType();

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'consent_manager_' . $this->getPluginType() . '.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'consent_manager_' . $this->getPluginType() . '_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#attributes']['class'][] = 'consent-manager-settings-form';
    $form['automatic'] = [
      '#type' => 'details',
      '#title' => $this->t('Install now'),
      '#open' => TRUE,
    ];

    $form['automatic']['install'] = [
      '#type' => 'button',
      '#value' => $this->t('Install now'),
      '#weight' => 100,
      '#attributes' => [
        'class' => [
          'button--primary',
          'consent-manager-install-button',
        ],
      ],
      '#attached' => [
        'library' => [
          'consent_manager/settings',
        ],
        'drupalSettings' => [
          'consent_manager' => [
            'type' => $this->getPluginType(),
            'domain' => $this->getRequest()->getHost(),
            'lang' => $this->languageManager->getDefaultLanguage()->getId(),
          ],
        ],
      ],
    ];

    $form['automatic']['instructions'] = [
      '#type' => 'item',
      '#markup' => $this->t('By clicking <strong>Install now</strong> you will be redirected to consentmanager.net and can follow our simple registration steps or login right away in order to choose your option. All details will then automatically be added here.'),
    ];

    $form['manual'] = [
      '#type' => 'details',
      '#title' => $this->t('Manual installation'),
      '#open' => TRUE,
    ];

    $form = parent::buildForm($form, $form_state);
    $form['actions']['submit']['#attributes']['class'][] = 'consent-manager-settings-form__submit';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $host = $form_state->getValue('host');
    if ($host && !filter_var($host, FILTER_VALIDATE_DOMAIN, FILTER_FLAG_HOSTNAME)) {
      $form_state->setErrorByName('host', $this->t('Wrong host value.'));
    }

    $cdn = $form_state->getValue('cdn');
    if ($cdn && !filter_var($cdn, FILTER_VALIDATE_DOMAIN, FILTER_FLAG_HOSTNAME)) {
      $form_state->setErrorByName('cdn', $this->t('Wrong CDN value.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    Cache::invalidateTags(['consent_manager_' . $this->getPluginType()]);
  }

}
